@@ -17,9 +17,15 @@ import torch.nn as nn
 import torch.nn.functional as F
 
 import poptorch
-import transformers
+from optimum.utils import logging
 from scipy.stats import truncnorm
-from transformers.utils import logging
+from transformers import (
+    BertForMultipleChoice,
+    BertForPreTraining,
+    BertForQuestionAnswering,
+    BertForSequenceClassification,
+    BertForTokenClassification,
+)
 
 from ...modeling_utils import PipelineMixin, register
 from .bert_fused_attention import BertFusedSelfAttention
@@ -111,7 +117,7 @@ def recomputation_checkpoint(module: nn.Module):
     def recompute_outputs(module, inputs, outputs):
         return tuple(poptorch.recomputationCheckpoint(y) for y in outputs)
 
-    module.register_forward_hook(recompute_outputs)
+    return module.register_forward_hook(recompute_outputs)
 
 
 def outline_attribute(module: nn.Module, value: str):
@@ -137,22 +143,20 @@ def outline_attribute(module: nn.Module, value: str):
     def disable(*args):
         context.__exit__(None, None, None)
 
-    module.register_forward_pre_hook(enable)
-    module.register_forward_hook(disable)
+    return module.register_forward_pre_hook(enable), module.register_forward_hook(disable)
 
 
-def accuracy(out, targ):
-    return (out.argmax(dim=-1) == targ).float().mean()
+@register(BertForPreTraining)
+class PipelinedBertForPreTraining(BertForPreTraining, PipelineMixin):
+    """
+    BertForPretraining transformed to run in an IPU pipeline.
 
+    Recommended usage:
+    ```
+    model = PipelinedBertForPretraining(config).parallelize().half().train()
+    ```
+    """
 
-def accuracy_masked(out, targ, mask_val):
-    mask = (targ != mask_val).float()
-    num_unmasked = mask.sum(1).unsqueeze(1)
-    return (out.argmax(dim=-1) == targ).float().mul(mask).div(num_unmasked).sum(1).mean()
-
-
-@register(transformers.BertForPreTraining)
-class PipelinedBertForPreTraining(transformers.BertForPreTraining, PipelineMixin):
     def __init__(self, config):
         super().__init__(config)
         self.gather_indices = OnehotGather()
@@ -164,12 +168,9 @@ class PipelinedBertForPreTraining(transformers.BertForPreTraining, PipelineMixin
         - Replaces self-attention layers with fused-qkv self-attention layers
         - (If enabled) Replaces the word embedding projection with a SerializedLinear layer
         - Adds recomputation checkpoints
-
-        Recommended usage:
-        ```
-        model = PipelinedBertForPretraining(config).parallelize().half().train()
-        ```
         """
+        super().parallelize()
+
         # Use faster fused-qkv self-attention
         for layer in self.bert.encoder.layer:
             fused = BertFusedSelfAttention(self.config)
@@ -195,12 +196,14 @@ class PipelinedBertForPreTraining(transformers.BertForPreTraining, PipelineMixin
         self.bert.embeddings = poptorch.BeginBlock(self.bert.embeddings, "Embedding", ipu_id=0)
         # Preventing the embeddings.LayerNorm from being outlined with the encoder.layer.LayerNorm
         # improves the tile mapping of the pipeline stashes
-        outline_attribute(self.bert.embeddings.LayerNorm, "embeddings")
+        hs = outline_attribute(self.bert.embeddings.LayerNorm, "embeddings")
+        self._hooks.extend(hs)
 
         for index, layer in enumerate(self.bert.encoder.layer):
             ipu = layer_ipu[index]
             if self.config.recompute_checkpoint_every_layer:
-                recomputation_checkpoint(layer)
+                h = recomputation_checkpoint(layer)
+                self._hooks.append(h)
             self.bert.encoder.layer[index] = poptorch.BeginBlock(layer, f"Encoder{index}", ipu_id=ipu)
             logger.info(f"Encoder {index:<2} --> IPU {ipu}")
 
@@ -248,19 +251,23 @@ class PipelinedBertForPreTraining(transformers.BertForPreTraining, PipelineMixin
         head_mask=None,
         inputs_embeds=None,
     ):
-        outputs = self.bert(input_ids=input_ids, attention_mask=attention_mask, token_type_ids=token_type_ids)
-        sequence_output, pooled_output = outputs[:2]
+        output = self.bert(input_ids=input_ids, attention_mask=attention_mask, token_type_ids=token_type_ids)
+        sequence_output, pooled_output = output[:2]
 
-        # Select only the masked tokens for the classifier
-        max_number_of_masked_tokens = int(labels.size(1) * 0.25)
-        masked_lm_labels, masked_lm_positions = torch.topk(labels, k=max_number_of_masked_tokens, dim=1)
-        masked_output = self.gather_indices(sequence_output, masked_lm_positions)
+        if labels is not None:
+            # Select only the masked tokens for the classifier
+            max_number_of_masked_tokens = int(labels.size(1) * 0.25)
+            masked_lm_labels, masked_lm_positions = torch.topk(labels, k=max_number_of_masked_tokens, dim=1)
+            masked_output = self.gather_indices(sequence_output, masked_lm_positions)
+        else:
+            # This case should never happen during training
+            masked_output = sequence_output
 
         prediction_scores, sequential_relationship_score = self.cls(masked_output, pooled_output)
-        outputs = (
+        output = (
             prediction_scores,
             sequential_relationship_score,
-        ) + outputs[2:]
+        ) + output[2:]
 
         if labels is not None and next_sentence_label is not None:
             masked_lm_loss = F.cross_entropy(
@@ -272,18 +279,9 @@ class PipelinedBertForPreTraining(transformers.BertForPreTraining, PipelineMixin
                 sequential_relationship_score.view(-1, 2), next_sentence_label.view(-1)
             ).float()
             total_loss = poptorch.identity_loss(masked_lm_loss + next_sentence_loss, reduction="none")
-            # next_sentence_acc = accuracy(sequential_relationship_score.view([-1, 2]), next_sentence_label.view(-1))
-            # labels: 0 if corresponding token not masked, original value otherwise
-            # masked_lm_acc = accuracy_masked(
-            #     prediction_scores.view([-1, self.config.mask_tokens, self.config.vocab_size]), labels, 0
-            # )
-            # masked_lm_acc = accuracy_masked(
-            #     prediction_scores, labels, 0
-            # )
-            # outputs = (total_loss, masked_lm_loss, next_sentence_loss, masked_lm_acc, next_sentence_acc)
-            outputs = (total_loss, masked_lm_loss, next_sentence_loss)
+            return (total_loss, masked_lm_loss, next_sentence_loss)
 
-        return outputs
+        return output
 
 
 class SerializedEmbedding(nn.Module):
@@ -350,8 +348,7 @@ class SerializedEmbedding(nn.Module):
         return x_sum
 
 
-@register(transformers.BertForQuestionAnswering)
-class PipelinedBertForQuestionAnswering(transformers.BertForQuestionAnswering, PipelineMixin):
+class BertPipelineMixin(PipelineMixin):
     def parallelize(self):
         """
         Transform the model to run in an IPU pipeline.
@@ -359,12 +356,9 @@ class PipelinedBertForQuestionAnswering(transformers.BertForQuestionAnswering, P
         - Replaces self-attention layers with fused-qkv self-attention layers
         - (If enabled) Replaces the word embedding with a SerializedEmbedding
         - Adds recomputation checkpoints
-
-        Recommended usage:
-        ```
-        model = PipelinedBertForQuestionAnswering(config).parallelize().half()
-        ```
         """
+        super().parallelize()
+
         # Use faster fused-qkv self-attention
         for layer in self.bert.encoder.layer:
             fused = BertFusedSelfAttention(self.config)
@@ -380,31 +374,137 @@ class PipelinedBertForQuestionAnswering(transformers.BertForQuestionAnswering, P
                 self.bert.embeddings.word_embeddings, self.config.embedding_serialization_factor
             )
         self.bert.embeddings = poptorch.BeginBlock(self.bert.embeddings, "Embedding", ipu_id=0)
-        outline_attribute(self.bert.embeddings.LayerNorm, "embedding")
+        hs = outline_attribute(self.bert.embeddings.LayerNorm, "embedding")
+        self._hooks.extend(hs)
 
         for index, layer in enumerate(self.bert.encoder.layer):
             ipu = layer_ipu[index]
             if self.config.recompute_checkpoint_every_layer and index != self.config.num_hidden_layers - 1:
-                recomputation_checkpoint(layer)
+                h = recomputation_checkpoint(layer)
+                self._hooks.append(h)
             self.bert.encoder.layer[index] = poptorch.BeginBlock(layer, f"Encoder{index}", ipu_id=ipu)
             logger.info(f"Encoder {index:<2} --> IPU {ipu}")
-
-        logger.info(f"QA Outputs --> IPU {ipu}")
-        self.qa_outputs = poptorch.BeginBlock(self.qa_outputs, "QA Outputs", ipu_id=ipu)
-        logger.info("-----------------------------------------------------------")
         return self
 
     def deparallelize(self):
         """
         Undo the changes to the model done by `parallelize`.
         You should call this before doing `save_pretrained` so that the `model.state_dict` is
-        fully compatible with `transformers.BertForQuestionAnswering`.
+        compatible with the original model.
         """
+        super().deparallelize()
         # Deserialize the serialized word embedding
         if self.config.embedding_serialization_factor > 1:
             self.bert.embeddings.word_embeddings = self.bert.embeddings.word_embeddings.deserialize()
         return self
 
+
+@register(BertForSequenceClassification)
+class PipelinedBertForSequenceClassification(BertForSequenceClassification, BertPipelineMixin):
+    """
+    BertForSequenceClassification transformed to run in an IPU pipeline.
+
+    Recommended usage:
+    ```
+    model = PipelinedBertForSequenceClassification(config).parallelize().half()
+    ```
+    """
+
+    def parallelize(self):
+        super().parallelize()
+        last_ipu = self.config.ipus_per_replica - 1
+        logger.info(f"Classifier Output --> IPU {last_ipu}")
+        self.classifier = poptorch.BeginBlock(self.classifier, "Classifier Output", ipu_id=last_ipu)
+        logger.info("-----------------------------------------------------------")
+        return self
+
+    def forward(self, input_ids, attention_mask, token_type_ids, labels=None):
+        return super().forward(
+            input_ids=input_ids,
+            attention_mask=attention_mask,
+            token_type_ids=token_type_ids,
+            labels=labels,
+            return_dict=False,
+        )
+
+
+@register(BertForMultipleChoice)
+class PipelinedBertForMultipleChoice(BertForMultipleChoice, BertPipelineMixin):
+    """
+    BertForMultipleChoice transformed to run in an IPU pipeline.
+
+    Recommended usage:
+    ```
+    model = PipelinedBertForMultipleChoice(config).parallelize().half()
+    ```
+    """
+
+    def parallelize(self):
+        super().parallelize()
+        last_ipu = self.config.ipus_per_replica - 1
+        logger.info(f"Classifier Output --> IPU {last_ipu}")
+        self.classifier = poptorch.BeginBlock(self.classifier, "Classifier Output", ipu_id=last_ipu)
+        logger.info("-----------------------------------------------------------")
+        return self
+
+    def forward(self, input_ids, attention_mask, token_type_ids, labels=None):
+        return super().forward(
+            input_ids=input_ids,
+            attention_mask=attention_mask,
+            token_type_ids=token_type_ids,
+            labels=labels,
+            return_dict=False,
+        )
+
+
+@register(BertForTokenClassification)
+class PipelinedBertForTokenClassification(BertForTokenClassification, BertPipelineMixin):
+    """
+    BertForTokenClassification transformed to run in an IPU pipeline.
+
+    Recommended usage:
+    ```
+    model = PipelinedBertForTokenClassification(config).parallelize().half()
+    ```
+    """
+
+    def parallelize(self):
+        super().parallelize()
+        last_ipu = self.config.ipus_per_replica - 1
+        logger.info(f"Classifier Output --> IPU {last_ipu}")
+        self.classifier = poptorch.BeginBlock(self.classifier, "Classifier Output", ipu_id=last_ipu)
+        logger.info("-----------------------------------------------------------")
+        return self
+
+    def forward(self, input_ids, attention_mask, token_type_ids, labels=None):
+        return super().forward(
+            input_ids=input_ids,
+            attention_mask=attention_mask,
+            token_type_ids=token_type_ids,
+            labels=labels,
+            return_dict=False,
+        )
+
+
+@register(BertForQuestionAnswering)
+class PipelinedBertForQuestionAnswering(BertForQuestionAnswering, BertPipelineMixin):
+    """
+    BertForQuestionAnswering transformed to run in an IPU pipeline.
+
+    Recommended usage:
+    ```
+    model = PipelinedBertForQuestionAnswering(config).parallelize().half()
+    ```
+    """
+
+    def parallelize(self):
+        super().parallelize()
+        last_ipu = self.config.ipus_per_replica - 1
+        logger.info(f"QA Outputs --> IPU {last_ipu}")
+        self.qa_outputs = poptorch.BeginBlock(self.qa_outputs, "QA Outputs", ipu_id=last_ipu)
+        logger.info("-----------------------------------------------------------")
+        return self
+
     def forward(self, input_ids, attention_mask, token_type_ids, start_positions=None, end_positions=None):
         output = super().forward(
             input_ids=input_ids,
@@ -412,9 +512,8 @@ class PipelinedBertForQuestionAnswering(transformers.BertForQuestionAnswering, P
             token_type_ids=token_type_ids,
             start_positions=start_positions,
             end_positions=end_positions,
+            return_dict=False,
         )
-        if self.training:
-            final_loss = poptorch.identity_loss(output.loss, reduction="none")
-            return final_loss, output.start_logits, output.end_logits
-        else:
-            return output.start_logits, output.end_logits
+        if start_positions is not None and end_positions is not None:
+            output = (poptorch.identity_loss(output[0], reduction="none"),) + output[1:]
+        return output
